package com.compo.camera.dialog;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.GridView;
import android.widget.ImageView;

import com.compo.camera.R;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by lmodlinski on 25/03/17.
 */
public class ColorPickerDialog extends DialogFragment
{
	protected static final String KEY_COLORS = "colors";
	protected static final String KEY_COLOR_SELECTED = "color_selected";
	protected static final float KEY_COLOR_DEGREE = 192 / 256;

	protected int[] colors = {};
	protected int color_selected;

	protected static final String KEY_COLUMNS = "columns";

	protected int columns = 3;

	protected int layoutId = R.layout.color_grid;

	protected ColorGridAdapter adapter;
	protected GridView grid;
	protected OnColorSelectedListener listener;

	public ColorPickerDialog()
	{
		super();
	}

	/**
	 * In-place factory method for creating instance of this dialog
	 *
	 * @param colors
	 * @param color
	 * @param columns
	 * @return
	 */
	public static ColorPickerDialog getInstance(int[] colors, int color, int columns)
	{
		ColorPickerDialog picker = new ColorPickerDialog();
		picker.initialize(colors, color, columns);

		return picker;
	}

	protected void setArguments(int columns)
	{
		Bundle bundle = new Bundle();
		bundle.putInt(KEY_COLUMNS, columns);

		this.setArguments(bundle);
	}

	protected void initialize(int[] colors, int color, int columns)
	{
		this.colors = colors;
		this.columns = columns;
		this.color_selected = color;

		this.setArguments(this.columns);
	}

	public void setOnColorSelectedListener(OnColorSelectedListener listener)
	{
		this.listener = listener;
	}

	protected void bind()
	{
		if (this.isAdded() && null == this.adapter) {
			this.adapter = new ColorGridAdapter();
		}

		if (null != this.adapter && null != this.grid) {
			this.adapter.setSelectedColor(this.color_selected);
			this.grid.setAdapter(this.adapter);
		}
	}

	protected static void setOverallColor(ImageView view, int color)
	{
		Drawable drawable = view.getDrawable();
		GradientDrawable gradient;

		if (null != drawable && drawable instanceof GradientDrawable) {
			gradient = (GradientDrawable) drawable;
		} else {
			gradient = new GradientDrawable();
			gradient.setShape(GradientDrawable.OVAL);
		}

		gradient.setColor(color);
		gradient.setStroke((int) TypedValue.applyDimension(
				TypedValue.COMPLEX_UNIT_DIP, 1, view.getContext().getResources().getDisplayMetrics()), Color.rgb(
				(int) (Color.red(color) * KEY_COLOR_DEGREE),
				(int) (Color.green(color) * KEY_COLOR_DEGREE),
				(int) (Color.blue(color) * KEY_COLOR_DEGREE)));

		view.setImageDrawable(gradient);
	}

	@Override
	public void onAttach(Activity activity)
	{
		super.onAttach(activity);

		if (0 < this.colors.length) {
			this.bind();
		}
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
	{
		View parent = super.onCreateView(inflater, container, savedInstanceState);

		this.getDialog().requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
		this.setStyle(DialogFragment.STYLE_NO_FRAME, android.R.style.Theme_Translucent);

		return parent;
	}

	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState)
	{
		View root = LayoutInflater.from(this.getActivity()).inflate(R.layout.color_dialog, null);

		if (null != this.getArguments()) {
			this.columns = this.getArguments().getInt(KEY_COLUMNS);
		}

		if (null != savedInstanceState) {
			this.colors = savedInstanceState.getIntArray(KEY_COLORS);
			this.color_selected = (int) savedInstanceState.getSerializable(KEY_COLOR_SELECTED);

			this.bind();
		}

		((ImageView) root.findViewById(R.id.color_background)).setImageResource(R.drawable.color_background);

		this.grid = (GridView) root.findViewById(R.id.color_dialog_id);
		this.grid.setNumColumns(this.columns);
		this.grid.setOnItemClickListener(new AdapterView.OnItemClickListener()
		{
			@Override
			public void onItemClick(AdapterView<?> listView, View view,
									int position, long itemId)
			{
				if (listener != null) {
					listener.onColorSelected(adapter.getItem(position));
				}

				dismiss();
			}
		});

		this.bind();

		return (new AlertDialog.Builder(this.getActivity()))
				.setView(root)
				.create();
	}

	@Override
	public void onSaveInstanceState(Bundle outState)
	{
		super.onSaveInstanceState(outState);

		outState.putIntArray(KEY_COLORS, this.colors);
		outState.putSerializable(KEY_COLOR_SELECTED, this.color_selected);
	}

	public interface OnColorSelectedListener
	{
		void onColorSelected(int color);
	}

	protected class ColorGridAdapter extends BaseAdapter
	{
		private List<Integer> choice = new ArrayList<Integer>();
		private int selected;

		private ColorGridAdapter()
		{
			for (int color : colors) {
				this.choice.add(color);
			}
		}

		@Override
		public int getCount()
		{
			return this.choice.size();
		}

		@Override
		public Integer getItem(int position)
		{
			return this.choice.get(position);
		}

		@Override
		public long getItemId(int position)
		{
			return this.choice.get(position);
		}

		@Override
		public View getView(int position, View convertView, ViewGroup container)
		{
			if (null == convertView) {
				convertView = LayoutInflater.from(getActivity()).inflate(layoutId, container, false);
			}

			int color = this.getItem(position);
			setOverallColor((ImageView) convertView.findViewById(R.id.color_view), color);

			this.setBorder((ImageView) convertView.findViewById(R.id.color_border), Color.WHITE);
			this.setBorder((ImageView) convertView.findViewById(R.id.color_frame), Color.rgb(136, 23, 5));

			convertView.findViewById(R.id.color_selected).setVisibility(color == selected ? View.VISIBLE : View.INVISIBLE);

			return convertView;
		}

		protected void setBorder(ImageView border, int color)
		{
			Drawable drawable = border.getDrawable();
			GradientDrawable gradient;

			if (null != drawable && drawable instanceof GradientDrawable) {
				gradient = (GradientDrawable) drawable;
			} else {
				gradient = new GradientDrawable();
				gradient.setShape(GradientDrawable.OVAL);
			}

			gradient.setColor(color);
			gradient.setStroke((int) TypedValue.applyDimension(
					TypedValue.COMPLEX_UNIT_DIP, 1,
					border.getContext().getResources().getDisplayMetrics()
			), color);

			border.setImageDrawable(gradient);
		}

		public void setSelectedColor(int color)
		{
			if (this.selected != color) {
				this.selected = color;

				this.notifyDataSetChanged();
			}
		}
	}
}
